package com.winhc.task.job;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;
import com.mongodb.client.model.UpdateOneModel;
import com.mongodb.client.model.UpdateOptions;
import com.mongodb.client.model.WriteModel;
import com.winhc.task.bean.CaseOut;
import com.winhc.task.bean.CompanyBackOut;
import com.winhc.task.framework.es.EsFastScan;
import com.winhc.task.framework.mongo.MongoDbFastScan;
import com.winhc.task.util.BaseUtils;
import com.winhc.task.util.EasyExcelUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import lombok.val;
import org.apache.commons.lang3.StringUtils;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.Get;
import org.bson.Document;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.search.SearchHit;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Component;

import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.function.Consumer;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * π
 */
@Slf4j
@Component
@AllArgsConstructor
public class CompanyBackUpdate {
    private final MongoTemplate mongoTemplate;

    @Autowired
    Connection connection;

    public void start() {
        MongoDatabase db = mongoTemplate.getDb();

        List<List<String>> head_list = Arrays.stream("公司名称@法人@联系电话@地址".split("@"))
                .map(Collections::singletonList)
                .collect(Collectors.toList());

        EasyExcelUtil company_shanghai = new EasyExcelUtil().init("D:\\tmp\\case\\" + "上海.xlsx", "上海", head_list, new HashSet<>());
        EasyExcelUtil company_guangzhou = new EasyExcelUtil().init("D:\\tmp\\case\\" + "广州.xlsx", "广州", head_list, new HashSet<>());
        EasyExcelUtil company_shengzheng = new EasyExcelUtil().init("D:\\tmp\\case\\" + "深圳.xlsx", "深圳", head_list, new HashSet<>());
        EasyExcelUtil company_suzhou = new EasyExcelUtil().init("D:\\tmp\\case\\" + "苏州.xlsx", "苏州", head_list, new HashSet<>());


        Consumer<List<Document>> func = list -> {
            List<CompanyBackOut> list1 = new ArrayList<>();
            List<CompanyBackOut> list2 = new ArrayList<>();
            List<CompanyBackOut> list3 = new ArrayList<>();
            List<CompanyBackOut> list4 = new ArrayList<>();
            list.forEach(d -> {
                JSONObject j = new JSONObject(d);
                String name = j.getString("name");
                String reg_location = j.getString("reg_location");
                String province_code = j.getString("province_code");
                String city_code = j.getString("city_code");

                String phones = Optional.ofNullable(j.getJSONArray("tel_info")).orElse(new JSONArray())
                        .toJavaList(JSONObject.class).stream()
                        .filter(x -> StringUtils.isNotBlank(x.getString("tel"))).map(x -> x.getString("tel"))
                        .distinct().collect(Collectors.joining("\n"));

                String legals = Optional.ofNullable(j.getJSONArray("lolegal_entities")).orElse(new JSONArray())
                        .toJavaList(JSONObject.class)
                        .stream()
                        .map(x -> {
                            JSONObject tj = new JSONObject();
                            tj.fluentPut("legal", x.getString("name"))
                                    .fluentPut("id", x.getString("id"))
                                    .fluentPut("legal_deleted", x.getInteger("deleted"));
                            return tj;
                        }).filter(x -> x.getString("legal_deleted").equals("0") && StringUtils.isNotBlank(x.getString("legal")))
                        .map(r1 -> r1.getString("legal"))
                        .collect(Collectors.joining("\n"));

                CompanyBackOut re = CompanyBackOut.builder()
                        .company_name(name)
                        .legal(legals)
                        .phones(phones)
                        .reg_location(reg_location)
                        .build();
                if (province_code.equals("31")) {
                    list1.add(re);

                } else if (province_code.equals("44") && city_code.equals("01")) {
                    list2.add(re);
                } else if (province_code.equals("44") && city_code.equals("03")) {
                    list3.add(re);
                } else if (province_code.equals("32") && city_code.equals("05")) {
                    list4.add(re);
                }
            });
            company_shanghai.doExportExcel(list1);
            company_guangzhou.doExportExcel(list2);
            company_shengzheng.doExportExcel(list3);
            company_suzhou.doExportExcel(list4);

        };

        MongoDbFastScan mongoDbFastScan = new MongoDbFastScan("xf_company_back_0728", func, db)
                .batchSize(200).threadNum(1);
        mongoDbFastScan.scan();

        company_shanghai.finish();
        company_guangzhou.finish();
        company_shengzheng.finish();
        company_suzhou.finish();
    }


}
