package com.winhc.task.job;

import com.alibaba.fastjson.JSONObject;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.model.InsertManyOptions;
import com.mongodb.client.result.InsertManyResult;
import com.winhc.task.framework.es.EsFastScan;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.bson.BsonValue;
import org.bson.Document;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.update.UpdateRequest;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.search.SearchHit;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;
import java.util.stream.Collectors;

import static com.winhc.task.util.DateUtils.randomDate;

/**
 * 合同模板 更新字段
 */
@Slf4j
@Component
@AllArgsConstructor
public class EsScanContractTemplateInfoJob {

    @Autowired
    @Qualifier(value = "v5")
    RestHighLevelClient getOldClient;

    public void start() {
        String dsl = "{\n" +
                "  \"query\": {\n" +
                "    \"match_all\": {}\n" +
                "  }\n" +
                "}";
        scan(dsl);
    }

    public void scan(String dsl) {

        Consumer<SearchHit[]> func = list -> {
            BulkRequest bulkRequest = new BulkRequest();
            Arrays.stream(list).forEach(d -> {
                Map<String, Object> m1 = d.getSourceAsMap();
                String id = d.getId();
                JSONObject data = new JSONObject(m1);

                String update_time = randomDate(2023, 2023, 3, 6);

                String content = data.getString("content");
                String re_content = content.replaceAll("合同法", "民法典")
                        .replaceAll("中华人民共和国合同法", "中华人民共和国民法典")
                        .replaceAll("婚姻法", "民法典")
                        .replaceAll("继承法", "民法典")
                        .replaceAll("民法通则", "民法典")
                        .replaceAll("收养法", "民法典")
                        .replaceAll("担保法", "民法典")
                        .replaceAll("物权法", "民法典")
                        .replaceAll("侵权责任法", "民法典")
                        .replaceAll("民法总则", "民法典");

                UpdateRequest request = new UpdateRequest("new_contract_template_info_v2", "new_contract_template_info_type", id);
                request.docAsUpsert(true);
                JSONObject rej = new JSONObject()
                        .fluentPut("update_time", update_time)
                        .fluentPut("content", re_content);
                request.doc(JSONObject.toJSONString(rej), XContentType.JSON);
                request.retryOnConflict(6);
                bulkRequest.add(request);
            });

            try {
                getOldClient.bulk(bulkRequest);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }

        };
        new EsFastScan(getOldClient, func, "new_contract_template_info_v2", "new_contract_template_info_type", dsl, null).scan();
    }

}
